@@ -100,6 +100,17 @@ PianoReturn_t PianoResponse (PianoHandle_t *ph, PianoRequest_t *req) {
 			ret = PIANO_RET_INVALID_RESPONSE;
 		} else {
 			ret = json_object_get_int (code)+PIANO_RET_OFFSET;
+
+			if (ret == PIANO_RET_P_INVALID_PARTNER_LOGIN &&
+					req->type == PIANO_REQUEST_LOGIN) {
+				PianoRequestDataLogin_t *reqData = req->data;
+				if (reqData->step == 1) {
+					/* return value is ambiguous, as both, partnerLogin and
+					 * userLogin return INVALID_PARTNER_LOGIN. Fix that to provide
+					 * better error messages. */
+					ret = PIANO_RET_INVALID_LOGIN;
+				}
+			}
 		}
 
 		json_object_put (j);
@@ -217,6 +228,7 @@ PianoReturn_t PianoResponse (PianoHandle_t *ph, PianoRequest_t *req) {
 
 			assert (req->responseData != NULL);
 			assert (reqData != NULL);
+			assert (reqData->quality != PIANO_AQ_UNKNOWN);
 
 			json_object *items = json_object_object_get (result, "items");
 			assert (items != NULL);
@@ -234,7 +246,37 @@ PianoReturn_t PianoResponse (PianoHandle_t *ph, PianoRequest_t *req) {
 					continue;
 				}
 
-				song->audioUrl = PianoJsonStrdup (s, "additionalAudioUrl");
+				/* get audio url based on selected quality */
+				static const char *qualityMap[] = {"", "lowQuality", "mediumQuality",
+						"highQuality"};
+				assert (reqData->quality < sizeof (qualityMap)/sizeof (*qualityMap));
+				static const char *formatMap[] = {"", "aacplus", "mp3"};
+				json_object *map = json_object_object_get (s, "audioUrlMap");
+				assert (map != NULL);
+
+				if (map != NULL) {
+					map = json_object_object_get (map, qualityMap[reqData->quality]);
+
+					if (map != NULL) {
+						const char *encoding = json_object_get_string (
+								json_object_object_get (map, "encoding"));
+						assert (encoding != NULL);
+						for (size_t k = 0; k < sizeof (formatMap)/sizeof (*formatMap); k++) {
+							if (strcmp (formatMap[k], encoding) == 0) {
+								song->audioFormat = k;
+								break;
+							}
+						}
+						song->audioUrl = PianoJsonStrdup (map, "audioUrl");
+					} else {
+						/* requested quality is not available */
+						ret = PIANO_RET_QUALITY_UNAVAILABLE;
+						free (song);
+						PianoDestroyPlaylist (playlist);
+						goto cleanup;
+					}
+				}
+
 				song->artist = PianoJsonStrdup (s, "artistName");
 				song->album = PianoJsonStrdup (s, "albumName");
 				song->title = PianoJsonStrdup (s, "songName");
@@ -244,7 +286,6 @@ PianoReturn_t PianoResponse (PianoHandle_t *ph, PianoRequest_t *req) {
 				song->detailUrl = PianoJsonStrdup (s, "songDetailUrl");
 				song->fileGain = json_object_get_double (
 						json_object_object_get (s, "trackGain"));
-				song->audioFormat = reqData->format;
 				switch (json_object_get_int (json_object_object_get (s,
 						"songRating"))) {
 					case 1:
@@ -293,21 +334,6 @@ PianoReturn_t PianoResponse (PianoHandle_t *ph, PianoRequest_t *req) {
 			break;
 		}
 
-		case PIANO_REQUEST_MOVE_SONG: {
-			/* move song to different station */
-			PianoRequestDataMoveSong_t *reqData = req->data;
-
-			assert (req->responseData != NULL);
-			assert (reqData != NULL);
-			assert (reqData->step < 2);
-
-			if (reqData->step == 0) {
-				ret = PIANO_RET_CONTINUE_REQUEST;
-				++reqData->step;
-			}
-			break;
-		}
-
 		case PIANO_REQUEST_DELETE_STATION: {
 			/* delete station from server and station list */
 			PianoStation_t *station = req->data;
@@ -654,6 +680,7 @@ PianoReturn_t PianoResponse (PianoHandle_t *ph, PianoRequest_t *req) {
 		}
 	}
 
+cleanup:
 	json_object_put (j);
 
 	return ret;
