@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2008-2011
+Copyright (c) 2008-2012
 	Lars-Dominik Braun <lars@6xq.net>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -31,6 +31,7 @@ THE SOFTWARE.
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
+/* fork () */
 #include <unistd.h>
 #include <sys/select.h>
 #include <time.h>
@@ -45,6 +46,11 @@ THE SOFTWARE.
 #include <assert.h>
 #include <stdbool.h>
 #include <limits.h>
+#include <signal.h>
+/* waitpid () */
+#include <sys/types.h>
+#include <sys/wait.h>
+
 
 /* pandora.com library */
 #include <piano.h>
@@ -91,22 +97,78 @@ static bool BarMainLoginUser (BarApp_t *app) {
 
 /*	ask for username/password if none were provided in settings
  */
-static void BarMainGetLoginCredentials (BarSettings_t *settings,
+static bool BarMainGetLoginCredentials (BarSettings_t *settings,
 		BarReadlineFds_t *input) {
 	if (settings->username == NULL) {
 		char nameBuf[100];
+
 		BarUiMsg (settings, MSG_QUESTION, "Email: ");
 		BarReadlineStr (nameBuf, sizeof (nameBuf), input, BAR_RL_DEFAULT);
 		settings->username = strdup (nameBuf);
 	}
+
 	if (settings->password == NULL) {
 		char passBuf[100];
-		BarUiMsg (settings, MSG_QUESTION, "Password: ");
-		BarReadlineStr (passBuf, sizeof (passBuf), input, BAR_RL_NOECHO);
-		/* write missing newline */
-		puts ("");
-		settings->password = strdup (passBuf);
+
+		if (settings->passwordCmd == NULL) {
+			BarUiMsg (settings, MSG_QUESTION, "Password: ");
+			BarReadlineStr (passBuf, sizeof (passBuf), input, BAR_RL_NOECHO);
+			/* write missing newline */
+			puts ("");
+			settings->password = strdup (passBuf);
+		} else {
+			pid_t chld;
+			int pipeFd[2];
+
+			BarUiMsg (settings, MSG_INFO, "Requesting password from external helper... ");
+
+			if (pipe (pipeFd) == -1) {
+				BarUiMsg (settings, MSG_NONE, "Error: %s\n", strerror (errno));
+				return false;
+			}
+
+			chld = fork ();
+			if (chld == 0) {
+				/* child */
+				close (pipeFd[0]);
+				dup2 (pipeFd[1], fileno (stdout));
+				execl ("/bin/sh", "/bin/sh", "-c", settings->passwordCmd, (char *) NULL);
+				BarUiMsg (settings, MSG_NONE, "Error: %s\n", strerror (errno));
+				close (pipeFd[1]);
+				exit (1);
+			} else if (chld == -1) {
+				BarUiMsg (settings, MSG_NONE, "Error: %s\n", strerror (errno));
+				return false;
+			} else {
+				/* parent */
+				int status;
+
+				close (pipeFd[1]);
+				memset (passBuf, 0, sizeof (passBuf));
+				read (pipeFd[0], passBuf, sizeof (passBuf)-1);
+				close (pipeFd[0]);
+
+				/* drop trailing newlines */
+				ssize_t len = strlen (passBuf)-1;
+				while (len >= 0 && passBuf[len] == '\n') {
+					passBuf[len] = '\0';
+					--len;
+				}
+
+				waitpid (chld, &status, 0);
+				if (WEXITSTATUS (status) == 0) {
+					settings->password = strdup (passBuf);
+					BarUiMsg (settings, MSG_NONE, "Ok.\n");
+				} else {
+					BarUiMsg (settings, MSG_NONE, "Error: Exit status %i.\n", WEXITSTATUS (status));
+					return false;
+				}
+			}
+
+		}
 	}
+
+	return true;
 }
 
 /*	get station list
@@ -163,7 +225,7 @@ static void BarMainGetPlaylist (BarApp_t *app) {
 	WaitressReturn_t wRet;
 	PianoRequestDataGetPlaylist_t reqData;
 	reqData.station = app->curStation;
-	reqData.format = app->settings.audioFormat;
+	reqData.quality = app->settings.audioQuality;
 
 	BarUiMsg (&app->settings, MSG_INFO, "Receiving new playlist... ");
 	if (!BarUiPianoCall (app, PIANO_REQUEST_GET_PLAYLIST,
@@ -206,6 +268,7 @@ static void BarMainStartPlayback (BarApp_t *app, pthread_t *playerThread) {
 		app->player.scale = BarPlayerCalcScale (app->player.gain + app->settings.volume);
 		app->player.audioFormat = app->playlist->audioFormat;
 		app->player.settings = &app->settings;
+		pthread_mutex_init (&app->player.pauseMutex, NULL);
 
 		/* throw event */
 		BarUiStartEventCmd (&app->settings, "songstart",
@@ -233,6 +296,7 @@ static void BarMainPlayerCleanup (BarApp_t *app, pthread_t *playerThread) {
 	/* FIXME: pthread_join blocks everything if network connection
 	 * is hung up e.g. */
 	pthread_join (*playerThread, &threadRet);
+	pthread_mutex_destroy (&app->player.pauseMutex);
 
 	/* don't continue playback if thread reports error */
 	if (threadRet != (void *) PLAYER_RET_OK) {
@@ -267,7 +331,9 @@ static void BarMainPrintTime (BarApp_t *app) {
 static void BarMainLoop (BarApp_t *app) {
 	pthread_t playerThread;
 
-	BarMainGetLoginCredentials (&app->settings, &app->input);
+	if (!BarMainGetLoginCredentials (&app->settings, &app->input)) {
+		return;
+	}
 
 	BarMainLoadProxy (&app->settings, &app->waith);
 
@@ -298,6 +364,7 @@ static void BarMainLoop (BarApp_t *app) {
 			if (app->playlist != NULL) {
 				PianoSong_t *histsong = app->playlist;
 				app->playlist = app->playlist->next;
+				histsong->next = NULL;
 				BarUiHistoryPrepend (app, histsong);
 			}
 			if (app->playlist == NULL) {
@@ -335,8 +402,14 @@ int main (int argc, char **argv) {
 	BarTermSetEcho (0);
 	BarTermSetBuffer (0);
 
+	/* signals */
+	signal (SIGPIPE, SIG_IGN);
+
 	/* init some things */
 	ao_initialize ();
+	gcry_check_version (NULL);
+	gcry_control (GCRYCTL_DISABLE_SECMEM, 0);
+	gcry_control (GCRYCTL_INITIALIZATION_FINISHED, 0);
 	gnutls_global_init ();
 
 	BarSettingsInit (&app.settings);
@@ -356,7 +429,8 @@ int main (int argc, char **argv) {
 	}
 
 	WaitressInit (&app.waith);
-	app.waith.url.host = strdup (PIANO_RPC_HOST);
+	app.waith.url.host = app.settings.rpcHost;
+	app.waith.url.tlsPort = app.settings.rpcTlsPort;
 	app.waith.tlsFingerprint = app.settings.tlsFingerprint;
 
 	/* init fds */
@@ -392,6 +466,9 @@ int main (int argc, char **argv) {
 		close (app.input.fds[1]);
 	}
 
+	/* write statefile */
+	BarSettingsWrite (app.curStation, &app.settings);
+
 	PianoDestroy (&app.ph);
 	PianoDestroyPlaylist (app.songHistory);
 	PianoDestroyPlaylist (app.playlist);
