@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2008-2011
+Copyright (c) 2008-2012
 	Lars-Dominik Braun <lars@6xq.net>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -28,6 +28,7 @@ THE SOFTWARE.
 #include <math.h>
 #include <stdint.h>
 #include <limits.h>
+#include <assert.h>
 #include <arpa/inet.h>
 
 #include "player.h"
@@ -135,17 +136,31 @@ static WaitressCbReturn_t BarPlayerAACCb (void *ptr, size_t size,
 		NeAACDecFrameInfo frameInfo;
 		size_t i;
 
-		while ((player->bufferFilled - player->bufferRead) >
-				player->sampleSize[player->sampleSizeCurr]) {
+		while (player->sampleSizeCurr < player->sampleSizeN &&
+				(player->bufferFilled - player->bufferRead) >=
+			player->sampleSize[player->sampleSizeCurr]) {
+			/* going through this loop can take up to a few seconds =>
+			 * allow earlier thread abort */
+			QUIT_PAUSE_CHECK;
+
 			/* decode frame */
 			aacDecoded = NeAACDecDecode(player->aacHandle, &frameInfo,
-					player->buffer + player->bufferRead,
+					&player->buffer[player->bufferRead],
 					player->sampleSize[player->sampleSizeCurr]);
+			player->bufferRead += player->sampleSize[player->sampleSizeCurr];
+			++player->sampleSizeCurr;
+
 			if (frameInfo.error != 0) {
+				/* skip this frame, songPlayed will be slightly off if this
+				 * happens */
 				BarUiMsg (player->settings, MSG_ERR, "Decoding error: %s\n",
 						NeAACDecGetErrorMessage (frameInfo.error));
-				break;
+				continue;
 			}
+			/* assuming data in stsz atom is correct */
+			assert (frameInfo.bytesconsumed ==
+					player->sampleSize[player->sampleSizeCurr-1]);
+
 			for (i = 0; i < frameInfo.samples; i++) {
 				aacDecoded[i] = applyReplayGain (aacDecoded[i], player->scale);
 			}
@@ -157,11 +172,10 @@ static WaitressCbReturn_t BarPlayerAACCb (void *ptr, size_t size,
 					(unsigned long long int) BAR_PLAYER_MS_TO_S_FACTOR /
 					(unsigned long long int) player->samplerate /
 					(unsigned long long int) player->channels;
-			player->bufferRead += frameInfo.bytesconsumed;
-			player->sampleSizeCurr++;
-			/* going through this loop can take up to a few seconds =>
-			 * allow earlier thread abort */
-			QUIT_PAUSE_CHECK;
+		}
+		if (player->sampleSizeCurr >= player->sampleSizeN) {
+			/* no more frames, drop data */
+			player->bufferRead = player->bufferFilled;
 		}
 	} else {
 		if (player->mode == PLAYER_INITIALIZED) {
@@ -235,12 +249,15 @@ static WaitressCbReturn_t BarPlayerAACCb (void *ptr, size_t size,
 				/* how many frames do we have? */
 				if (player->sampleSizeN == 0) {
 					/* mp4 uses big endian, convert */
+					memcpy (&player->sampleSizeN, player->buffer +
+							player->bufferRead, sizeof (uint32_t));
 					player->sampleSizeN =
-							bigToHostEndian32 (*((uint32_t *) (player->buffer +
-							player->bufferRead)));
+							bigToHostEndian32 (player->sampleSizeN);
+
 					player->sampleSize = malloc (player->sampleSizeN *
 							sizeof (*player->sampleSize));
-					player->bufferRead += 4;
+					assert (player->sampleSize != NULL);
+					player->bufferRead += sizeof (uint32_t);
 					player->sampleSizeCurr = 0;
 					/* set up song duration (assuming one frame always contains
 					 * the same number of samples)
@@ -253,11 +270,15 @@ static WaitressCbReturn_t BarPlayerAACCb (void *ptr, size_t size,
 							(unsigned long long int) player->channels;
 					break;
 				} else {
+					memcpy (&player->sampleSize[player->sampleSizeCurr],
+							player->buffer + player->bufferRead,
+							sizeof (uint32_t));
 					player->sampleSize[player->sampleSizeCurr] =
-							bigToHostEndian32 (*((uint32_t *) (player->buffer +
-							player->bufferRead)));
+							bigToHostEndian32 (
+							player->sampleSize[player->sampleSizeCurr]);
+
 					player->sampleSizeCurr++;
-					player->bufferRead += 4;
+					player->bufferRead += sizeof (uint32_t);
 				}
 				/* all sizes read, nearly ready for data mode */
 				if (player->sampleSizeCurr >= player->sampleSizeN) {
@@ -421,7 +442,6 @@ void *BarPlayerThread (void *data) {
 	WaitressReturn_t wRet = WAITRESS_RET_ERR;
 
 	/* init handles */
-	pthread_mutex_init (&player->pauseMutex, NULL);
 	player->waith.data = (void *) player;
 	/* extraHeaders will be initialized later */
 	player->waith.extraHeaders = extraHeaders;
@@ -429,7 +449,6 @@ void *BarPlayerThread (void *data) {
 
 	switch (player->audioFormat) {
 		#ifdef ENABLE_FAAD
-		case PIANO_AF_AACPLUS_LO:
 		case PIANO_AF_AACPLUS:
 			player->aacHandle = NeAACDecOpen();
 			/* set aac conf */
@@ -444,7 +463,6 @@ void *BarPlayerThread (void *data) {
 
 		#ifdef ENABLE_MAD
 		case PIANO_AF_MP3:
-		case PIANO_AF_MP3_HI:
 			mad_stream_init (&player->mp3Stream);
 			mad_frame_init (&player->mp3Frame);
 			mad_synth_init (&player->mp3Synth);
@@ -473,7 +491,6 @@ void *BarPlayerThread (void *data) {
 
 	switch (player->audioFormat) {
 		#ifdef ENABLE_FAAD
-		case PIANO_AF_AACPLUS_LO:
 		case PIANO_AF_AACPLUS:
 			NeAACDecClose(player->aacHandle);
 			free (player->sampleSize);
@@ -482,7 +499,6 @@ void *BarPlayerThread (void *data) {
 
 		#ifdef ENABLE_MAD
 		case PIANO_AF_MP3:
-		case PIANO_AF_MP3_HI:
 			mad_synth_finish (&player->mp3Synth);
 			mad_frame_finish (&player->mp3Frame);
 			mad_stream_finish (&player->mp3Stream);
@@ -500,7 +516,6 @@ void *BarPlayerThread (void *data) {
 
 	ao_close(player->audioOutDevice);
 	WaitressFree (&player->waith);
-	pthread_mutex_destroy (&player->pauseMutex);
 	free (player->buffer);
 
 	player->mode = PLAYER_FINISHED_PLAYBACK;
