@@ -38,6 +38,7 @@ THE SOFTWARE.
 /* required for freebsd */
 #include <sys/types.h>
 #include <pthread.h>
+#include <stdint.h>
 
 #include <piano.h>
 #include <waitress.h>
@@ -45,6 +46,7 @@ THE SOFTWARE.
 #include "settings.h"
 
 #define BAR_PLAYER_MS_TO_S_FACTOR 1000
+#define BAR_PLAYER_BUFSIZE (WAITRESS_BUFFER_SIZE*2)
 
 struct audioPlayer {
 	char doQuit;
@@ -82,7 +84,7 @@ struct audioPlayer {
 	/* stsz atom: sample sizes */
 	size_t sampleSizeN;
 	size_t sampleSizeCurr;
-	unsigned int *sampleSize;
+	uint32_t *sampleSize;
 	NeAACDecHandle aacHandle;
 	#endif
 
@@ -97,10 +99,10 @@ struct audioPlayer {
 	ao_device *audioOutDevice;
 	const BarSettings_t *settings;
 
+	unsigned char *buffer;
+
 	pthread_mutex_t pauseMutex;
 	WaitressHandle_t waith;
-	/* buffer; should be large enough */
-	unsigned char buffer[WAITRESS_BUFFER_SIZE*2];
 };
 
 enum {PLAYER_RET_OK = 0, PLAYER_RET_ERR = 1};
